﻿#ifndef _EVENTMANAGER_H
#define _EVENTMANAGER_H

#include <list>

class IEventFunctionImp
{
public:
     virtual int Run(bool arg) = 0;
};

template<class T>
class EventFunctionImp : public IEventFunctionImp
{
public:
    typedef int (T::*Callback)(bool arg);

    EventFunctionImp(T * obj, Callback callback) : m_pObj(obj), m_callback(callback)
    {
    }

    int Run(bool arg)
    {
         return (m_pObj->*m_callback)(arg);
    }

private:
     T * m_pObj;
     Callback m_callback;
};


class EventBase
{
public:
    float m_duration;
    float m_elapse;
	IEventFunctionImp* m_pFunc;

public:
	EventBase(float duration);
	void SetCallback(IEventFunctionImp* func);
    virtual bool IsDone();
	virtual void Stop();
    virtual void Advance(float dt);
};

class EventManager
{
private:
    std::list<EventBase*> m_events;
public:
	EventManager();
	void AddEvent(EventBase* ievent);
	void Update(float dt);
};   

#endif